(define-module (ide edit)
  #:use-module (ide command)
  #:use-module (ide keyboard)
  #:use-module (ide interface)
  #:use-module (ide wx bindings))

;;TODO (when text_p ...)
(define-interactive (zoom-in)
  "Zoom in."
  (key-command STC-ZOOMIN))

(define-interactive (zoom-out)
  "Zoom out."
  (key-command STC-ZOOMOUT))

(define-interactive (kill-line)
  "Kill line."
  (if (= 0 (line-len)) (key-command STC-LINEDOWNEXTEND)
      (key-command STC-LINEENDEXTEND))
  (key-command STC-CUT))

(define-interactive (yank)
  "Reinsert (\"paste\") the last stretch of killed text."
  (key-command STC-PASTE))

(define-interactive (backward-char #:optional (n 1))
  "Move character backward.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (if (marked) (key-command STC-CHARLEFTEXTEND)
        (key-command STC-CHARLEFT))))

(define-interactive (forward-char #:optional (n 1))
  "Move character forward.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (if (marked) (key-command STC-CHARRIGHTEXTEND)
        (key-command STC-CHARRIGHT))))

(define-interactive (previous-line #:optional (n 1))
  "Move line up.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (if (marked) (key-command STC-LINEUPEXTEND)
        (key-command STC-LINEUP))))

(define-interactive (next-line #:optional (n 1))
  "Move line down.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (if (marked) (key-command STC-LINEDOWNEXTEND)
        (key-command STC-LINEDOWN))))

(define-interactive (backward-word #:optional (n 1))
  "Move backward until encountering the beginning of a word.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n)) (key-command STC-WORDLEFT)))

(define-interactive (forward-word #:optional (n 1))
  "Move forward until encountering the end of a word.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n)) (key-command STC-WORDRIGHT)))

(define-interactive (beginning-of-buffer)
  "Move to beginning of buffer."
  (key-command STC-DOCUMENTSTART))

(define-interactive (end-of-buffer)
  "Move to end of buffer."
  (key-command STC-DOCUMENTEND))

(define-interactive (undo)
  "Undo."
  (key-command STC-UNDO))

(define-interactive (delete-char #:optional (n 1))
  "Delete the following character."
  (do ((i 0 (1+ i))) ((= i n))
    (key-command STC-CHARRIGHTEXTEND)
    (key-command STC-CUT)))

(define-interactive (delete-char #:optional (n 1))
  "Delete the following character."
  (do ((i 0 (1+ i))) ((= i n))
    (key-command STC-CHARRIGHTEXTEND)
    (key-command STC-CUT)))

(define-interactive (kill-word #:optional (n 1))
  "Kill characters forward until encountering the end of a word.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (key-command STC-WORDRIGHTENDEXTEND)
    (key-command STC-CUT)))

(define-interactive (backward-kill-word #:optional (n 1))
  "Kill characters backward until encountering the beginning of a word.

With argument N, do this that many times."
  (do ((i 0 (1+ i))) ((= i n))
    (key-command STC-WORDRIGHTENDEXTEND)
    (key-command STC-CUT)))

(define-interactive (kill-region)
  "Kill (\"cut\") text between point and mark."
  (key-command STC-CUT))

(define-interactive (copy-region)
  "Copy text between point and mark."
  (key-command STC-COPY))

(define-interactive (upcase-word)
  "Copy text between point and mark."
  (key-command STC-WORDRIGHTENDEXTEND)
  (key-command STC-UPPERCASE)
  ;;(dzn-clear)?
  )

(define-interactive (downcase-word)
  "Copy text between point and mark."
  (key-command STC-WORDRIGHTENDEXTEND)
  (key-command STC-LOWERCASE)
  ;;(dzn-clear)
)

(define-interactive (beginning-of-line)
  "Move to beginning of line."
  (if (marked) (key-command STC-HOMEEXTEND)
      (key-command STC-HOME)))

(define-interactive (end-of-line)
  "Move to end of line."
  (if (marked) (key-command STC-LINEENDEXTEND)
      (key-command STC-LINEEND)))

(define-interactive (what-cursor-position)
  "Show cursor position."
  (message (format #f "~a" (point)))
  (point))

(define-interactive (isearch-forward)
  (minibuffer-prefix "I-search: ")) ;; TODO?

(define-interactive (open-line)
  (let ((here (point)))
    (insert "\n")
    (move-point here)))
