(define-module (ide help)
  #:use-module (srfi srfi-26)
  #:use-module (ide command)
  #:use-module (ide keyboard)
  #:use-module (ide minibuffer)
  #:use-module (ide interface)
  #:export (%prefix))

(define %prefix (dirname (dirname (current-filename))))

(define-interactive (help-bury)
  "bury help buffer"
  (when (help-visible?) (help-toggle)))

(define-interactive (goto-url url)
  "Open URL in browser."
  (unless (help-visible?)
    (help-toggle))
  (visit-url url))

(define-interactive (info)
  (goto-url (string-append "file://" %prefix "/info/index.html")))

(define-interactive (help)
  (goto-url (string-append "file://" %prefix "/info/help.html")))

(define-interactive (help-explore)
  (goto-url (string-append "file://" (getenv "HOME") "/.npm/lib/node_modules/dzn-enterprise/root/help.html")))

(define-interactive (help-bindings)
  (goto-url (string-append "file://" %prefix (if (emacs-key-map?) "/info/emacs-keys.html"
                                                 "/info/windows-keys.html"))))

(define (describe-procedure completer)
  (set! global-key-map minibuffer-key-map)
  (set! current-key-map minibuffer-key-map)
  (set! minibuffer-complete completer)
  (set! minibuffer-return
        (lambda _
          (let* ((command (minibuffer-string))
                 (doc (string-append %prefix "/info/command/" command ".html")))
            (minibuffer-clear)
            (minibuffer-done)
            (if (access? doc R_OK) (goto-url (string-append "file://" doc))
                (message (format #f "No documentation for ~a" command))))))
  (minibuffer-prefix "Describe function: "))

(define-interactive (describe-function)
  (describe-procedure ide-procedure-completer))

(define-interactive (describe-command)
  (describe-procedure command-completer))

(define-interactive (describe-key)
  (goto-url (string-append "file://" %prefix "/info/command/emacs-mode.html")))
