(define-module (ide keyboard)
  #:use-module (srfi srfi-26)

  #:use-module (ide interface)
  #:use-module (ide command)
  #:use-module (ide console)
  #:use-module (ide minibuffer)
  #:use-module (ide wx bindings)
  #:use-module (ide wx modifiers)
  #:export (key-map?
            global-key-map
            define-key
            ctrl-x-map
            ctrl-h-map
            console-key-map
            minibuffer-key-map
            emacs-global-key-map
            windows-global-key-map
            current-key-map
            after-cua-mode-hook
            kbd
            keycode
            set-console-key-map
            set-default-key-map
            set-minibuffer-key-map
            define-key
            emacs-key-map?))

(define-public STC-META STC-ALT)

;; FIXME?
(define-public STC-SHIFT-KEY 306)
(define-public STC-CTRL-KEY 307)
(define-public STC-META-KEY 308)
(define-public STC-SUPER-KEY 308)

;;points momentarily to the actual key map
(define current-key-map #f)

;; These are set after reading user init
(define global-key-map #f)
(define console-key-map #f)
(define minbuffer-key-map #f)
(define ctrl-h-map #f)
(define ctrl-x-map #f)

(define emacs-global-key-map)
(define windows-global-key-map)

(define (keycode key . mod)
  (+ (apply + mod)
     (ash (if (char? key) (char->integer key) key) 8)))

(define (key-map? x)
  (list? x))

(define (define-key key-map key-sequence procedure-or-keymap)
  "In KEY-MAP, define KEY-SEQUENCE as PROCEDURE-OR-KEYMAP.

KEY-SEQUENCE is a string that describes one or more key strokes.  E.g.

(define (find-file-at-point)
  (let* ((start (point))
         (str (buffer-substring start (point-max)))
         (char-set:file-name (string->char-set \"/.-+\" char-set:letter+digit))
         (file-name-boundary (char-set-complement char-set:file-name))
         (end (or (string-index str file-name-boundary)
                  (string-length str)))
         (file-at-point (substring str 0 end)))
    (if (and (string? file-at-point) (not (string-null? file-at-point))) (visit-file file-at-point)
        (find-file))))

(define-key global-key-map \"C-z C-f\" find-file-at-point)
" ;; help paredit "
  (let ((key key-sequence)
        (keys (string-split key-sequence #\space)))
    (cond ((= (length keys) 1)
           (let ((old (map identity key-map)))
             (set-car! key-map `(,key . ,procedure-or-keymap))
             (set-cdr! key-map old)))
          ((= (length keys) 2)
           (let* ((pre-map-key (car keys))
                  (sub-key-map (assoc-ref key-map pre-map-key))
                  (create? (not (key-map? sub-key-map))))
             (when create?
               (set! sub-key-map `((,"keymap" . #t))))
             (define-key sub-key-map (string-join (cdr keys) " ") procedure-or-keymap)
             (when create?
               (define-key key-map pre-map-key sub-key-map)))))))

(define (kbd mod+key)
  (cond ((string= mod+key "<left>")
         (keycode 314))
        ((string= mod+key "<up>")
         (keycode 315))
        ((string= mod+key "<right>")
         (keycode 316))
        ((string= mod+key "<down>")
         (keycode 317))
        ((string= mod+key "<f1>")
	 (keycode 340))
        ((string= mod+key "<f2>")
	 (keycode 341))
        ((string= mod+key "<f3>")
	 (keycode 342))
        ((string= mod+key "<f4>")
	 (keycode 343))
        ((string= mod+key "<f5>")
	 (keycode 344))
        ((string= mod+key "<f5>")
	 (keycode 344))
        ((string= mod+key "<f6>")
	 (keycode 345))
        ((string= mod+key "<f7>")
	 (keycode 346))
        ((string= mod+key "<f8>")
	 (keycode 347))
        ((string= mod+key "<f10>")
	 (keycode 349))
        ((string= mod+key "<f11>")
	 (keycode 350))
        ((string= mod+key "<f12>")
	 (keycode 351))
        ((string= mod+key "C--")
	 (+ STC-CTRL (keycode #\-)))
        ((string= mod+key "C-+")
	 (+ (+ STC-SHIFT STC-CTRL) (keycode #\+)))
        ((string= mod+key "C-=")
	 (+ STC-CTRL (keycode #\=)))
        ((string= mod+key "C-_")
	 (+ (+ STC-SHIFT STC-CTRL) (keycode #\_)))
        ((string= mod+key "M-:")
	 (+ (+ STC-SHIFT STC-META) (keycode #\;)))
        ((string= mod+key "M-.")
	 (+ (+ STC-SHIFT STC-META) (keycode #\.)))
        ((string= mod+key "M-<")
	 (+ (+ STC-SHIFT STC-META) (keycode #\<)))
        ((string= mod+key "M-!")
	 (+ (+ STC-SHIFT STC-META) (keycode #\1)))
        ((string= mod+key "M-&")
	 (+ (+ STC-SHIFT STC-META) (keycode #\7)))
	((string= mod+key "C-SPACE")
	 (+ STC-CTRL (keycode #\space)))
	((string= mod+key "M-DEL")
	 (+ STC-META (keycode #\backspace)))
        ((string= mod+key "M-<f4>")
	 (+ STC-META (keycode 343)))
	((string= mod+key "SPACE")
	 (keycode #\space))
	((string= mod+key "RET")
	 (keycode #\return))
	((string= mod+key "TAB")
	 (keycode #\tab))
        ((string= mod+key "ESC")
	 (keycode #\escape))
	(else
	 (let ((modifiers `((#\S . ,STC-SHIFT)
			    (#\C . ,STC-CTRL)
			    (#\M . ,STC-META)
			    (#\s . ,STC-SUPER))))

	   (let* ((key+mod (reverse (string-split mod+key #\-)))
		  (key (char-upcase (string-ref (car key+mod) 0)))
		  (mod (map (compose (cut assoc-ref modifiers <>) (cut string-ref <> 0)) (cdr key+mod))))
	     (apply keycode (cons key mod)))))))

(define (emacs-key-map?)
  (eq? global-key-map
       emacs-global-key-map))

(define-interactive (cua-mode #:optional arg)
  "Toggle Common User Access style editing (CUA mode)."
  (if  (or (and (number? arg) (positive? arg))
             (and (boolean? arg) arg)
             (and (not arg) (emacs-key-map?)))
      (begin
        (message "windows mode")
        (set! global-key-map windows-global-key-map))
      (begin
        (message "emacs mode")
        (set! global-key-map emacs-global-key-map)))
  (after-cua-mode-hook))

(define-interactive (emacs-mode)
  "Switch to emacs mode."
  (cua-mode -1))

(define (after-cua-mode-hook)
  (set! current-key-map global-key-map)
  (set! minibuffer-key-map
        `(("RET" . ,(lambda _ (minibuffer-return)))
          ("TAB" . ,(lambda _ (minibuffer-complete)))
          ("SPACE" . ,(lambda _ (minibuffer-complete)))
          ,@global-key-map))
  (set! console-key-map
        `(("RET" . ,(lambda _ (console-return)))
          ("TAB" . ,(lambda _ #f)) ;;COMPLETION
          ("C-c" . ,(lambda _ (console-interrupt)))
          ("C-l" . ,(lambda _ (console-reset)))
          ("C-u" . ,(lambda _ (console-clear-line)))
          ,@global-key-map)))

(define (set-console-key-map)
  (set! current-key-map console-key-map))

(define (set-minibuffer-key-map)
  (set! current-key-map minibuffer-key-map))

(define (set-default-key-map)
  (set! current-key-map global-key-map))
